@@ -1,6 +1,6 @@
 /*
 
-  $Id: cfgreader.c,v 1.74 2006/06/17 20:00:01 pkot Exp $
+  $Id: cfgreader.c,v 1.79 2007/07/21 13:01:24 dforsi Exp $
 
   G N O K I I
 
@@ -36,8 +36,6 @@
 
 */
 
-#define GNOKII_DEPRECATED /* do not warn about deprecated functions here */
-
 #include "config.h"
 #include "compat.h"
 #include "misc.h"
@@ -406,13 +404,16 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 
 	memset(cfg, '\0', sizeof(gn_config));
 
+	/* You need to specify at least model and port in the phone section */
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "model")))
-		strcpy(cfg->model, def->model);
+		/* strcpy(cfg->model, def->model); */
+		return false;
 	else
 		snprintf(cfg->model, sizeof(cfg->model), "%s", val);
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "port")))
-		strcpy(cfg->port_device, def->port_device);
+		/* strcpy(cfg->port_device, def->port_device); */
+		return false;
 	else
 		snprintf(cfg->port_device, sizeof(cfg->port_device), "%s", val);
 
@@ -433,7 +434,8 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 			cfg->init_length = 0;
 		else if (sscanf(val, " %d %c", &cfg->init_length, &ch) != 1) {
 			fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "initlength", val);
-			return false;
+			fprintf(stderr, _("Assuming: %d\n"), def->init_length);
+			cfg->init_length = def->init_length;
 		}
 	}
 
@@ -441,14 +443,16 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 		cfg->serial_baudrate = def->serial_baudrate;
 	else if (sscanf(val, " %d %c", &cfg->serial_baudrate, &ch) != 1) {
 		fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "serial_baudrate", val);
-		return false;
+		fprintf(stderr, _("Assuming: %d\n"), def->serial_baudrate);
+		cfg->serial_baudrate = def->serial_baudrate;
 	}
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "serial_write_usleep")))
 		cfg->serial_write_usleep = def->serial_write_usleep;
 	else if (sscanf(val, " %d %c", &cfg->serial_write_usleep, &ch) != 1) {
 		fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "serial_write_usleep", val);
-		return false;
+		fprintf(stderr, _("Assuming: %d\n"), def->serial_write_usleep);
+		cfg->serial_write_usleep = def->serial_write_usleep;
 	}
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "handshake")))
@@ -458,16 +462,18 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 	else if (!strcasecmp(val, "hardware") || !strcasecmp(val, "xonxoff"))
 		cfg->hardware_handshake = true;
 	else {
-		fprintf(stderr, _("Unrecognized [%s] option \"%s\", use \"%s\" or \"%s\" value\n"),
-				 section, "handshake", "software", "hardware");
-		return false;
+		fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "handshake", val);
+		fprintf(stderr, _("Use either \"%s\" or \"%s\".\n"), "software", "hardware");
+		fprintf(stderr, _("Assuming: %s\n"), def->hardware_handshake ? "software" : "hardware");
+		cfg->hardware_handshake = def->hardware_handshake;
 	}
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "require_dcd")))
 		cfg->require_dcd = def->require_dcd;
 	else if (sscanf(val, " %d %c", &cfg->require_dcd, &ch) != 1) {
 		fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "require_dcd", val);
-		return false;
+		fprintf(stderr, _("Assuming: %d\n"), def->require_dcd);
+		cfg->require_dcd = def->require_dcd;
 	}
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "smsc_timeout")))
@@ -476,7 +482,8 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 		cfg->smsc_timeout *= 10;
 	else {
 		fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "smsc_timeout", val);
-		return false;
+		fprintf(stderr, _("Assuming: %d\n"), def->smsc_timeout);
+		cfg->smsc_timeout = def->smsc_timeout;
 	}
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "connect_script")))
@@ -493,14 +500,16 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 		cfg->rfcomm_cn = def->rfcomm_cn;
 	else if (sscanf(val, " %hhu %c", &cfg->rfcomm_cn, &ch) != 1) {
 		fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "rfcomm_channel", val);
-		return false;
+		fprintf(stderr, _("Assuming: %d\n"), def->rfcomm_cn);
+		cfg->rfcomm_cn = def->rfcomm_cn;
 	}
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "sm_retry")))
 		cfg->sm_retry = def->sm_retry;
 	else if (sscanf(val, " %d %c", &cfg->sm_retry, &ch) != 1) {
 		fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "sm_retry", val);
-		return false;
+		fprintf(stderr, _("Assuming: %d\n"), def->sm_retry);
+		cfg->sm_retry = def->sm_retry;
 	}
 
 	/* There is no global setting. You need to set irda_string
@@ -512,6 +521,19 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 		dprintf("Setting irda_string in section %s to %s\n", section, cfg->irda_string);
 	}
 
+	if (!(val = gn_cfg_get(gn_cfg_info, section, "use_locking")))
+		cfg->use_locking = def->use_locking;
+	else if (!strcasecmp(val, "no"))
+		cfg->use_locking = 0;
+	else if (!strcasecmp(val, "yes"))
+		cfg->use_locking = 1;
+	else {
+		fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "use_locking", val);
+		fprintf(stderr, _("Use either \"%s\" or \"%s\".\n"), "yes", "no");
+		fprintf(stderr, _("Assuming: %s\n"), def->use_locking ? "yes" : "no");
+		cfg->use_locking = def->use_locking;
+	}
+
 	return true;
 }
 
@@ -527,15 +549,17 @@ static bool cfg_get_log_target(gn_log_target *t, const char *opt)
 	else if (!strcasecmp(val, "on"))
 		*t = GN_LOG_T_STDERR;
 	else {
-		fprintf(stderr, _("Unrecognized [logging] option \"%s\", use \"off\" or \"on\" value\n"), opt);
-		return false;
+		fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), "logging", opt, val);
+		fprintf(stderr, _("Use either \"%s\" or \"%s\".\n"), "off", "on");
+		fprintf(stderr, _("Assuming: %s\n"), "off");
+		*t = GN_LOG_T_NONE;
 	}
 
 	return true;
 }
 
 #define MAX_PATH_LEN 200
-GNOKII_API int gn_cfg_read_default()
+GNOKII_API gn_error gn_cfg_read_default()
 {
 	char *homedir;
 	char rcfile[MAX_PATH_LEN];
@@ -566,16 +590,16 @@ GNOKII_API int gn_cfg_read_default()
 		if (gn_cfg_file_read(globalrc)) {
 			/* That failed too so exit */
 			fprintf(stderr, _("Couldn't open %s or %s.\n"), rcfile, globalrc);
-			return -1;
+			return GN_ERR_NOCONFIG;
 		}
 	}
-	return 0;
+	return GN_ERR_NONE;
 }
 
 /* DEPRECATED */
-GNOKII_API int gn_cfg_read(char **bindir)
+GNOKII_API gn_error gn_cfg_read(char **bindir)
 {
-	int retval;
+	gn_error retval;
 
 	retval = gn_cfg_read_default();
 
@@ -588,13 +612,13 @@ GNOKII_API int gn_cfg_read(char **bindir)
 	return retval;
 }
 
-static int cfg_file_or_memory_read(const char *file, const char **lines)
+static gn_error cfg_file_or_memory_read(const char *file, const char **lines)
 {
 	char *val;
 
 	if (file == NULL && lines == NULL) {
 		dprintf("Couldn't open a config file or memory.\n");
-		return -1;
+		return GN_ERR_NOCONFIG;
 	}
 
 	/* I know that it doesn't belong here but currently there is now generic
@@ -613,8 +637,11 @@ static int cfg_file_or_memory_read(const char *file, const char **lines)
 
 	if (gn_cfg_info == NULL) {
 		/* this is bad, but the previous was much worse - bozo */
-		dprintf("Couldn't open %s config file,\n", file);
-		return -1;
+		if (file)
+			dprintf("Couldn't open %s config file.\n", file);
+		else
+			dprintf("Couldn't read config\n");
+		return GN_ERR_NOCONFIG;
 	}
 	strcpy(gn_config_default.model, "");
 	strcpy(gn_config_default.port_device, "");
@@ -630,10 +657,11 @@ static int cfg_file_or_memory_read(const char *file, const char **lines)
 	strcpy(gn_config_default.disconnect_script, "");
 	gn_config_default.rfcomm_cn = 1;
 	gn_config_default.sm_retry = 0;
+	gn_config_default.use_locking = 0;
 
 	if (!cfg_psection_load(&gn_config_global, "global", &gn_config_default)) {
 		fprintf(stderr, _("No [global] section in %s config file.\n"), file);
-		return -2;
+		return GN_ERR_NOPHONE;
 	}
 
 	/* hack to support [sms] / smsc_timeout parameter */
@@ -644,27 +672,30 @@ static int cfg_file_or_memory_read(const char *file, const char **lines)
 			gn_config_global.smsc_timeout = 10 * atoi(val);
 	}
 
-	if (!cfg_get_log_target(&gn_log_debug_mask, "debug")) return -2;
-	if (!cfg_get_log_target(&gn_log_rlpdebug_mask, "rlpdebug")) return -2;
-	if (!cfg_get_log_target(&gn_log_xdebug_mask, "xdebug")) return -2;
+	if (!cfg_get_log_target(&gn_log_debug_mask, "debug") ||
+	    !cfg_get_log_target(&gn_log_rlpdebug_mask, "rlpdebug") ||
+	    !cfg_get_log_target(&gn_log_xdebug_mask, "xdebug"))
+		return GN_ERR_NOLOG;
 
 	gn_log_debug("LOG: debug mask is 0x%x\n", gn_log_debug_mask);
 	gn_log_rlpdebug("LOG: rlpdebug mask is 0x%x\n", gn_log_rlpdebug_mask);
 	gn_log_xdebug("LOG: xdebug mask is 0x%x\n", gn_log_xdebug_mask);
-	return 0;
+	if (file)
+		dprintf("Config read from file %s.\n", file);
+	return GN_ERR_NONE;
 }
 
-GNOKII_API int gn_cfg_file_read(const char *file)
+GNOKII_API gn_error gn_cfg_file_read(const char *file)
 {
 	return cfg_file_or_memory_read(file, NULL);
 }
 
-GNOKII_API int gn_cfg_memory_read(const char **lines)
+GNOKII_API gn_error gn_cfg_memory_read(const char **lines)
 {
 	return cfg_file_or_memory_read(NULL, lines);
 }
 
-GNOKII_API int gn_cfg_phone_load(const char *iname, struct gn_statemachine *state)
+GNOKII_API gn_error gn_cfg_phone_load(const char *iname, struct gn_statemachine *state)
 {
 	char section[256];
 
@@ -673,17 +704,17 @@ GNOKII_API int gn_cfg_phone_load(const char *iname, struct gn_statemachine *stat
 	} else {
 		snprintf(section, sizeof(section), "phone_%s", iname);
 		if (!cfg_psection_load(&state->config, section, &gn_config_global))
-			return false;
+			return GN_ERR_NOPHONE;
 	}
 
 	if (state->config.model[0] == '\0') {
 		fprintf(stderr, _("Config error - no model specified.\n"));
-		return false;
+		return GN_ERR_NOMODEL;
 	}
 	if (state->config.port_device[0] == '\0') {
 		fprintf(stderr, _("Config error - no port specified.\n"));
-		return false;
+		return GN_ERR_NOPORT;
 	}
 
-	return true;
+	return GN_ERR_NONE;
 }
