@@ -1,6 +1,6 @@
 /*
 
-  $Id: at-emulator.c,v 1.44 2005/12/30 00:13:21 pkot Exp $
+  $Id: at-emulator.c,v 1.47 2007/07/05 21:59:05 pkot Exp $
 
   G N O K I I
 
@@ -42,7 +42,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <time.h>
-#include <unistd.h>
 #include <ctype.h>
 
 #ifndef WIN32
@@ -52,6 +51,7 @@
 #include "config.h"
 #include "misc.h"
 #include "gnokii.h"
+#include "compat.h"
 #include "data/at-emulator.h"
 #include "data/datapump.h"
 
@@ -218,7 +218,7 @@ static void  gn_atem_answer_phone(void)
 
 
 /* This gets called to indicate an incoming call */
-void gn_atem_call_passup(gn_call_status CallStatus, gn_call_info *CallInfo, struct gn_statemachine *state)
+void gn_atem_call_passup(gn_call_status CallStatus, gn_call_info *CallInfo, struct gn_statemachine *state, void *callback_data)
 {
 	dprintf("gn_atem_call_passup called with %d\n", CallStatus);
 
@@ -397,7 +397,7 @@ void	gn_atem_at_parse(char *cmd_buffer)
 			CommandMode = false;
 			if (gn_sm_functions(GN_OP_MakeCall, &data, sm) != GN_ERR_NONE) {
 				CommandMode = true;
-				dp_CallPassup(GN_CALL_RemoteHangup, NULL, NULL);
+				dp_CallPassup(GN_CALL_RemoteHangup, NULL, NULL, NULL);
 			} else {
 				IncomingCallNo = data.call_info->call_id;
 				gn_sm_loop(10, sm);
