@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-sms.c,v 1.149 2006/06/11 16:39:48 deller Exp $
+  $Id: gsm-sms.c,v 1.163 2007/07/15 09:17:32 dforsi Exp $
 
   G N O K I I
 
@@ -26,7 +26,8 @@
   Copyright (C) 2001-2002 Markus Plail, Pavel Machek <pavel@ucw.cz>
   Copyright (C) 2002-2003 Ladis Michl
   Copyright (C) 2003-2004 BORBELY Zoltan
-
+  Copyright (C) 2006      Igor Popik
+  
   Library for parsing and creating Short Messages (SMS).
 
 */
@@ -134,13 +135,13 @@ GNOKII_API void gn_sms_default_deliver(gn_sms *sms)
  * sms_timestamp_print - formats date and time in the human readable manner
  * @number: binary coded date and time
  *
- * The function converts binary date time into a easily readable form.
+ * The function converts binary date time into an easily readable form.
  * It is Y2K compliant.
  */
 static char *sms_timestamp_print(u8 *number)
 {
 #ifdef DEBUG
-#define LOCAL_DATETIME_MAX_LENGTH 23
+#define LOCAL_DATETIME_MAX_LENGTH 26
 	static char buffer[LOCAL_DATETIME_MAX_LENGTH];
 	char buf[5];
 	int i;
@@ -157,8 +158,14 @@ static char *sms_timestamp_print(u8 *number)
 		sprintf(buffer, "19");
 
 	for (i = 0; i < 6; i++) {
+		int c;
 		char buf2[4];
-		snprintf(buf2, 4, "%d%d-", number[i] & 0x0f, number[i] >> 4);
+		switch (i) {
+		case 0: case 1: c = '-'; break;
+		case 3: case 4: c = ':'; break;
+		default: c = ' '; break;
+		}
+		snprintf(buf2, 4, "%d%d%c", number[i] & 0x0f, number[i] >> 4, c);
 		strcat(buffer, buf2);
 	}
 
@@ -402,7 +409,7 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
  * @udhlen: udh length 
  * @dcs: data coding scheme
  *
- * This function decodes either 7bit or 8bit os Unicode text to the
+ * This function decodes either 7bit or 8bit or Unicode text to the
  * readable text format according to the locale set.
  */
 static gn_error sms_data_decode(unsigned char *message, unsigned char *output, unsigned int length,
@@ -430,11 +437,11 @@ static gn_error sms_data_decode(unsigned char *message, unsigned char *output, u
 }
 
 /**
- * sms_udh_decode - interprete the User Data Header contents
+ * sms_udh_decode - interprets the User Data Header contents
  * @message: received UDH
  * @sms: SMS structure to fill in
  *
- * At this stage we already need to know thet SMS has UDH.
+ * At this stage we already need to know that SMS has UDH.
  * This function decodes UDH as described in:
  *   - GSM 03.40 version 6.1.0 Release 1997, section 9.2.3.24
  *   - Smart Messaging Specification, Revision 1.0.0, September 15, 1997
@@ -458,6 +465,13 @@ static gn_error sms_udh_decode(unsigned char *message, gn_sms_udh *udh)
 			udh->udh[nr].u.concatenated_short_message.maximum_number   = message[pos + 3];
 			udh->udh[nr].u.concatenated_short_message.current_number   = message[pos + 4];
 			break;
+		case 0x08: /* Concatenated short messages, 16-bit reference number */
+			dprintf("Concatenated messages, 16-bit reference number\n");
+			udh->udh[nr].type = GN_SMS_UDH_ConcatenatedMessages;
+			udh->udh[nr].u.concatenated_short_message.reference_number = 256 * message[pos + 2] + message[pos + 3];
+			udh->udh[nr].u.concatenated_short_message.maximum_number   = message[pos + 4];
+			udh->udh[nr].u.concatenated_short_message.current_number   = message[pos + 5];
+			break;
 		case 0x01: /* Special SMS Message Indication */
 			switch (message[pos + 2] & 0x03) {
 			case 0x00:
@@ -480,7 +494,7 @@ static gn_error sms_udh_decode(unsigned char *message, gn_sms_udh *udh)
 			udh->udh[nr].u.special_sms_message_indication.store = (message[pos + 2] & 0x80) >> 7;
 			udh->udh[nr].u.special_sms_message_indication.message_count = message[pos + 3];
 			break;
-		case 0x05: /* Application port addression scheme, 16 bit address */
+		case 0x05: /* Application port addressing scheme, 16 bit address */
 			switch (((0x00ff & message[pos + 2]) << 8) | (0x00ff & message[pos + 3])) {
 			case 0x1581:
 				dprintf("Ringtone\n");
@@ -524,7 +538,7 @@ static gn_error sms_udh_decode(unsigned char *message, gn_sms_udh *udh)
 				break;
 			}
 			break;
-		case 0x04: /* Application port addression scheme, 8 bit address */
+		case 0x04: /* Application port addressing scheme, 8 bit address */
 		case 0x06: /* SMSC Control Parameters */
 		case 0x07: /* UDH Source Indicator */
 		default:
@@ -545,13 +559,13 @@ static gn_error sms_udh_decode(unsigned char *message, gn_sms_udh *udh)
 }
 
 /**
- * sms_header_decode - Doecodes PDU SMS header
+ * sms_header_decode - Decodes PDU SMS header
  * @rawsms:
  * @sms:
  * @udh:
  *
  * This function parses received SMS header information and stores
- * them in hihger level SMS struct. It also checks for the UDH and when
+ * them in higher level SMS struct. It also checks for the UDH and when
  * it's found calls the function to extract the UDH.
  */
 static gn_error sms_header_decode(gn_sms_raw *rawsms, gn_sms *sms, gn_sms_udh *udh)
@@ -608,7 +622,7 @@ static gn_error sms_header_decode(gn_sms_raw *rawsms, gn_sms *sms, gn_sms_udh *u
 	/* User Data Header */
 	if (rawsms->udh_indicator & 0x40) { /* UDH header available */
 		dprintf("UDH found\n");
-		sms_udh_decode(rawsms->user_data, udh);
+		return sms_udh_decode(rawsms->user_data, udh);
 	}
 
 	return GN_ERR_NONE;
@@ -677,12 +691,11 @@ static gn_error sms_pdu_decode(gn_sms_raw *rawsms, gn_sms *sms)
 		break;
 	/* Plain text message */
 	default:
-		size = rawsms->length - sms->udh.length;
-		sms_data_decode(rawsms->user_data + sms->udh.length,        /* Skip the UDH */
+		return sms_data_decode(rawsms->user_data + sms->udh.length, /* Skip the UDH */
 				(unsigned char *)&sms->user_data[0].u.text, /* With a plain text message we have only 1 part */
-				rawsms->length,                            /* Length of the decoded text */
-				size,                                      /* Length of the encoded text (in full octets) without UDH */
-				sms->udh.length,                          /* To skip the certain number of bits when unpacking 7bit message */
+				rawsms->length,                             /* Length of the decoded text */
+				rawsms->user_data_length,                   /* Length of the encoded text (in full octets) without UDH */
+				sms->udh.length,                            /* To skip the certain number of bits when unpacking 7bit message */
 				sms->dcs);
 		break;
 	}
@@ -706,12 +719,12 @@ GNOKII_API gn_error gn_sms_parse(gn_data *data)
 }
 
 /**
- * gn_sms_request - High-level function for the expicit SMS reading from the phone
+ * gn_sms_request - High-level function for the explicit SMS reading from the phone
  * @data: GSM data for the phone driver
  * @state: current statemachine state
  *
  * This is the function for explicit requesting the SMS from the
- * phone driver. Not that raw_sms field in the gn_data structure must
+ * phone driver. Note that raw_sms field in the gn_data structure must
  * be initialized
  */
 GNOKII_API gn_error gn_sms_request(gn_data *data, struct gn_statemachine *state)
@@ -725,7 +738,7 @@ GNOKII_API gn_error gn_sms_request(gn_data *data, struct gn_statemachine *state)
  * @data: GSM data for the phone driver
  * @state: current statemachine state
  *
- * This function is the frotnend for reading SMS. Note that SMS field
+ * This function is the frontend for reading SMS. Note that SMS field
  * in the gn_data structure must be initialized.
  */
 GNOKII_API gn_error gn_sms_get(gn_data *data, struct gn_statemachine *state)
@@ -733,7 +746,12 @@ GNOKII_API gn_error gn_sms_get(gn_data *data, struct gn_statemachine *state)
 	gn_error error;
 	gn_sms_raw rawsms;
 
-	if (!data->sms) return GN_ERR_INTERNALERROR;
+	if (!data->sms)
+		return GN_ERR_INTERNALERROR;
+	if (data->sms->number < 1)
+		return GN_ERR_EMPTYLOCATION;
+	if (data->sms->memory_type > GN_MT_LAST)
+		return GN_ERR_INVALIDMEMORYTYPE;
 	memset(&rawsms, 0, sizeof(gn_sms_raw));
 	rawsms.number = data->sms->number;
 	rawsms.memory_type = data->sms->memory_type;
@@ -745,7 +763,7 @@ GNOKII_API gn_error gn_sms_get(gn_data *data, struct gn_statemachine *state)
 }
 
 /**
- * gn_sms_delete - High-level function for delting SMS
+ * gn_sms_delete - High-level function for deleting SMS
  * @data: GSM data for the phone driver
  * @state: current statemachine state
  *
@@ -883,7 +901,7 @@ static gn_error sms_verify_status(gn_data *data)
 				(data->message_list[i][data->sms_folder->folder_id]->status == GN_SMS_FLD_NotReadHandled)) {
 			for (j = 0; j < data->sms_folder->number; j++) {
 				if (data->message_list[i][data->sms_folder->folder_id]->location == data->sms_folder->locations[j]) {
-					/* We have a found a formerly unread message which has been read in the meantime */
+					/* We have found a formerly unread message which has been read in the meantime */
 					dprintf("Found a formerly unread message which has been read in the meantime: loc: %i\n",
 							data->message_list[i][data->sms_folder->folder_id]->location);
 					data->message_list[i][data->sms_folder->folder_id]->status = GN_SMS_FLD_Changed;
@@ -1044,7 +1062,7 @@ static gn_error sms_concat_header_encode(gn_sms_raw *rawsms, int curr, int total
  * @clen: coded data length
  *
  * This function does the phone frame encoding basing on the given SMS
- * structure. This function is capable to create only one frame at a time.
+ * structure. This function is capable of creating only one frame at a time.
  */
 static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 {
@@ -1131,26 +1149,28 @@ static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 			unsigned int length, udh_length, offset = rawsms->user_data_length;
 
 			length = strlen(sms->user_data[i].u.text);
+			if (sms->udh.length)
+				udh_length = sms->udh.length + 1;
+			else
+				udh_length = 0;
 			switch (al) {
 			case GN_SMS_DCS_DefaultAlphabet:
-				if (sms->udh.length) udh_length = sms->udh.length+1;
-				else udh_length = 0;
 				size = char_7bit_pack((7 - (udh_length % 7)) % 7,
 						      sms->user_data[i].u.text,
 						      rawsms->user_data + offset,
 						      &length);
-				rawsms->length = length;
+				rawsms->length = length + (udh_length * 8 + 6) / 7;
 				rawsms->user_data_length = size + offset;
 				break;
 			case GN_SMS_DCS_8bit:
 				rawsms->dcs |= 0xf4;
 				memcpy(rawsms->user_data + offset, sms->user_data[i].u.text, sms->user_data[i].u.text[0]);
-				rawsms->user_data_length = rawsms->length = length + offset;
+				rawsms->user_data_length = rawsms->length = length + udh_length;
 				break;
 			case GN_SMS_DCS_UCS2:
 				rawsms->dcs |= 0x08;
 				length = char_unicode_encode(rawsms->user_data + offset, sms->user_data[i].u.text, length);
-				rawsms->user_data_length = rawsms->length = length + offset;
+				rawsms->user_data_length = rawsms->length = length + udh_length;
 				break;
 			default:
 				return GN_ERR_WRONGDATAFORMAT;
@@ -1236,7 +1256,7 @@ gn_error sms_prepare(gn_sms *sms, gn_sms_raw *rawsms)
 	}
 	/* Encoding the header */
 	rawsms->report = sms->delivery_report;
-	rawsms->remote_number[0] = char_semi_octet_pack(sms->remote.number, rawsms->remote_number + 1, sms->remote.type);
+		rawsms->remote_number[0] = char_semi_octet_pack(sms->remote.number, rawsms->remote_number + 1, sms->remote.type);
 	rawsms->validity_indicator = GN_SMS_VP_RelativeFormat;
 	rawsms->validity[0] = 0xa9;
 
@@ -1244,9 +1264,8 @@ gn_error sms_prepare(gn_sms *sms, gn_sms_raw *rawsms)
 		if (sms->udh.udh[i].type == GN_SMS_UDH_ConcatenatedMessages)
 			sms_concat_header_encode(rawsms, sms->udh.udh[i].u.concatenated_short_message.current_number,
 						sms->udh.udh[i].u.concatenated_short_message.maximum_number);
-	sms_data_encode(sms, rawsms);
 
-	return GN_ERR_NONE;
+	return sms_data_encode(sms, rawsms);
 }
 
 static void sms_dump_raw(gn_sms_raw *rawsms)
@@ -1294,7 +1313,7 @@ GNOKII_API gn_error gn_sms_send(gn_data *data, struct gn_statemachine *state)
 		data->raw_sms->message_center[0] = data->raw_sms->message_center[0] / 2 + 1;
 
 	error = sms_prepare(data->sms, data->raw_sms);
-	ERROR();
+	if (error != GN_ERR_NONE) goto cleanup;
 
 	sms_dump_raw(data->raw_sms);
 	if (data->raw_sms->user_data_length > MAX_SMS_PART) {
@@ -1321,27 +1340,31 @@ GNOKII_API int gn_sms_udh_add(gn_sms *sms, gn_sms_udh_type type)
 
 static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 {
-	int i, count, start, copied, total, isConcat = -1;
+	int i, count, start, copied, total, isConcat = -1, max_sms_len = MAX_SMS_PART;
 	gn_sms sms;
 	gn_sms_user_data ud[GN_SMS_PART_MAX_NUMBER];
 	gn_error error = GN_ERR_NONE;
 
+	/* count -- number of SMS to be sent
+	 * total -- total number of octets to be sent
+	 */
 	sms = *data->sms;
 
 	/* If there's no concat header we need to add one */
 	for (i = 0; i < data->sms->number; i++)
-		if (data->sms->udh.udh[i].type == GN_SMS_UDH_ConcatenatedMessages) isConcat = i;
+		if (data->sms->udh.udh[i].type == GN_SMS_UDH_ConcatenatedMessages)
+			isConcat = i;
 
 	if (isConcat == -1)
 		isConcat = gn_sms_udh_add(data->sms, GN_SMS_UDH_ConcatenatedMessages);
 
-	/* Count the total length of the octets to be sent */
+	/* Count the total length of the message text octets to be sent */
 	total = 0;
 	i = 0;
 	while (data->sms->user_data[i].type != GN_SMS_DATA_None) {
 		switch (data->sms->dcs.u.general.alphabet) {
 		case GN_SMS_DCS_DefaultAlphabet:
-			total += ((data->sms->udh.length+1) % 8 + data->sms->user_data[i].length) * 7 / 8;
+			total += (data->sms->user_data[i].length * 7 + 7) / 8;
 			break;
 		case GN_SMS_DCS_UCS2:
 			total += (data->sms->user_data[i].length * 2);
@@ -1353,10 +1376,11 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 		memcpy(&ud[i], &data->sms->user_data[i], sizeof(gn_sms_user_data));
 		i++;
 	}
-	total += (data->sms->udh.length+1);
 
+	/* We need to attach user data header to each part */
+	max_sms_len -= (data->sms->udh.length + 1);
 	/* Count number of SMS to be sent */
-	count = (total + MAX_SMS_PART - 1)/ MAX_SMS_PART;
+	count = (total + max_sms_len - 1)/ max_sms_len;
 	dprintf("Will need %d sms-es\n", count);
 
 	start = 0;
@@ -1369,9 +1393,11 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 		switch (data->sms->dcs.u.general.alphabet) {
 		case GN_SMS_DCS_DefaultAlphabet:
 			start += copied;
-			copied = (ud[0].length - start) % ((MAX_SMS_PART - (data->sms->udh.length+1)) * 8 / 7);
-			if (copied == 0)
-				copied = ((MAX_SMS_PART - (data->sms->udh.length+1)) * 8 / 7);
+			if (ud[0].length - start >= max_sms_len * 8 / 7) {
+				copied = max_sms_len * 8 / 7;
+			} else {
+				copied = (ud[0].length - start) % (max_sms_len * 8 / 7);
+			}
 			memset(&data->sms->user_data[0], 0, sizeof(gn_sms_user_data));
 			data->sms->user_data[0].type = ud[0].type;
 			data->sms->user_data[0].length = copied;
@@ -1379,16 +1405,19 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 			break;
 		case GN_SMS_DCS_UCS2:
 			start += copied;
-			copied = (ud[0].length - start) % ((MAX_SMS_PART - (data->sms->udh.length+1)));
-			if (copied == 0)
-				copied = (MAX_SMS_PART - (data->sms->udh.length+1));
-			if (copied > (MAX_SMS_PART - (data->sms->udh.length+1)) / 2) {
+			copied = (ud[0].length - start) % max_sms_len;
+			/* This is maximum size of what we can send */
+			if (copied == 0) {
+				copied = max_sms_len;
+			}
+			if (copied > max_sms_len / 2) {
 				copied = copied / 2;
-				if (copied < (MAX_SMS_PART - (data->sms->udh.length+1)) / 2)
-					copied = (MAX_SMS_PART - (data->sms->udh.length+1)) / 2;
-
+				if (copied < max_sms_len / 2)
+					copied = max_sms_len / 2;
+			}
+			if (i < count - 1 && copied % 2) {
+				copied--;
 			}
-			dprintf("%d\n", copied);
 			memset(&data->sms->user_data[0], 0, sizeof(gn_sms_user_data));
 			data->sms->user_data[0].type = ud[0].type;
 			data->sms->user_data[0].length = copied;
@@ -1451,3 +1480,145 @@ cleanup:
 	data->raw_sms = NULL;
 	return error;
 }
+
+/* WAPPush functions */
+
+char *encode_attr_inline_string(char token, char *string, int *data_len) 
+{
+	char *data = NULL;
+	
+	/* we need 3 extra bytes for tags */
+	*data_len = strlen(string) + 3;
+	data = malloc(*data_len);
+
+	if (!data) {
+	    return NULL;
+	}
+	
+	data[0] = token;
+	data[1] = TAG_INLINE;
+	memcpy(data + 2, string, strlen(string));
+	data[*data_len - 1] = 0x00;
+
+	return data;
+}
+
+char *encode_indication(gn_wap_push *wp, int *data_len) 
+{
+	char *data = NULL;
+	char *attr = NULL;
+	int attr_len = 0;
+	int offset = 0;
+	
+	/* encode tag attribute */
+	attr = encode_attr_inline_string(ATTR_HREF, wp->url, &attr_len);
+	
+	if (!attr || !attr_len) {
+	    return NULL;
+	}
+	
+	/* need 5 extra bytes for indication token & attributes */
+	*data_len = attr_len + strlen(wp->text) + 5;
+	data = malloc(*data_len);
+	
+	if (!data) {
+	    return NULL;
+	}
+	
+	/* indication tag token */
+	data[offset++] = TOKEN_KNOWN_AC | TAG_INDICATION;
+	
+	/* attribute */
+	memcpy(data + offset, attr, attr_len);
+	offset += attr_len;
+	data[offset++] = TAG_END;
+
+	/* wappush text */
+	data[offset++] = TAG_INLINE;
+	memcpy(data + offset, wp->text, strlen(wp->text));
+	offset += strlen(wp->text);
+	data[offset++] = 0x00;
+	
+	/* tag end */
+	data[offset++] = TAG_END;
+
+	free(attr);
+
+	return data;
+}
+
+char *encode_si(gn_wap_push *wp, int *data_len)
+{
+	char *data = NULL;
+	char *child = NULL;
+	int child_len = 0;
+	
+	child = encode_indication(wp, &child_len);
+
+	if (!child || !data_len) {
+	    return NULL;
+	}
+
+	/* we need extra 2 bytes for si token */
+	*data_len = child_len + 2;
+	data = malloc(*data_len);
+	
+	if (!data) {
+	    free(child);
+	    return NULL;
+	}
+	
+	data[0] = TOKEN_KNOWN_C | TAG_SI;
+	memcpy(data + 1, child, child_len);
+	data[*data_len - 1] = TAG_END;
+	
+	free(child);
+	
+	return data;
+}
+
+GNOKII_API gn_error gn_wap_push_encode(gn_wap_push *wp) 
+{
+	
+	char *data = NULL;
+	int data_len = 0;
+	
+	data = encode_si(wp, &data_len);
+	
+	if (!data || !data_len) {
+	    return GN_ERR_FAILED;
+	}
+	
+	wp->data = malloc(data_len + sizeof(gn_wap_push_header));
+
+	if (!wp->data) {
+	    return GN_ERR_FAILED;
+	}
+	
+	memcpy(wp->data, &wp->header, sizeof(gn_wap_push_header));
+	memcpy(wp->data + sizeof(gn_wap_push_header), data, data_len);
+	
+	wp->data_len = data_len + sizeof(gn_wap_push_header);
+
+	return GN_ERR_NONE;
+}
+
+GNOKII_API void gn_wap_push_init(gn_wap_push *wp) 
+{
+
+	if (!wp) {
+	    return;
+	}
+
+	memset(wp, 0, sizeof(gn_wap_push));
+	
+	wp->header.wsp_tid 		= 0x00;
+	wp->header.wsp_pdu 		= PDU_TYPE_Push;
+	wp->header.wsp_hlen 		= 0x01;
+	wp->header.wsp_content_type 	= CONTENT_TYPE;
+
+	wp->header.version 	= WBXML_VERSION;
+	wp->header.public_id 	= TAG_SI;
+	wp->header.charset 	= WAPPush_CHARSET;
+	wp->header.stl 		= 0x00; /* string table length */
+}
