@@ -1,6 +1,6 @@
 /*
 
-  $Id: atgen.c,v 1.126 2006/08/08 22:06:39 pkot Exp $
+  $Id: atgen.c,v 1.153 2007/07/11 19:34:29 pkot Exp $
 
   G N O K I I
 
@@ -29,6 +29,7 @@
   Copyright (C) 2002-2004 BORBELY Zoltan
   Copyright (C) 2003-2004 Igor Popik
   Copyright (C) 2004      Hugo Hass, Ron Yorston
+  Copyright (C) 2007      Jeremy Laine
 
   This file provides functions specific to generic AT command compatible
   phones. See README for more details on supported mobile phones.
@@ -48,6 +49,7 @@
 #include "phones/atnok.h"
 #include "phones/atsie.h"
 #include "phones/atsoer.h"
+#include "phones/atsam.h"
 #include "links/atbus.h"
 
 static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state);
@@ -68,9 +70,15 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 /* static gn_error ReplyDeleteSMS(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state); */
 static gn_error ReplyGetCharset(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
 static gn_error ReplyGetSMSCenter(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
+#ifdef SECURITY
 static gn_error ReplyGetSecurityCodeStatus(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
+#endif
 static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
 static gn_error ReplyRing(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
+static gn_error ReplyGetDateTime(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
+static gn_error ReplyGetActiveCalls(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
+static gn_error ReplyIncomingSMS(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
+static gn_error ReplyGetSMSMemorySize(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state);
 
 static gn_error AT_Identify(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetModel(gn_data *data, struct gn_statemachine *state);
@@ -95,13 +103,21 @@ static gn_error AT_DeleteSMS(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetCharset(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_SetCharset(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetSMSCenter(gn_data *data, struct gn_statemachine *state);
+#ifdef SECURITY
 static gn_error AT_EnterSecurityCode(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetSecurityCodeStatus(gn_data *data, struct gn_statemachine *state);
+#endif
 static gn_error AT_DialVoice(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_GetNetworkInfo(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_AnswerCall(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_CancelCall(gn_data *data, struct gn_statemachine *state);
 static gn_error AT_SetCallNotification(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_SetDateTime(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_GetDateTime(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_SendDTMF(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_GetActiveCalls(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_OnSMS(gn_data *data, struct gn_statemachine *state);
+static gn_error AT_GetSMSMemorySize(gn_data *data, struct gn_statemachine *state);
 
 typedef struct {
 	int gop;
@@ -137,14 +153,23 @@ static at_function_init_type at_function_init[] = {
 	{ GN_OP_AT_GetCharset,         AT_GetCharset,            ReplyGetCharset },
 	{ GN_OP_AT_SetCharset,         AT_SetCharset,            Reply },
 	{ GN_OP_GetSMSCenter,          AT_GetSMSCenter,          ReplyGetSMSCenter },
+#ifdef SECURITY
 	{ GN_OP_GetSecurityCodeStatus, AT_GetSecurityCodeStatus, ReplyGetSecurityCodeStatus },
 	{ GN_OP_EnterSecurityCode,     AT_EnterSecurityCode,     Reply },
+#endif
 	{ GN_OP_MakeCall,              AT_DialVoice,             Reply },
 	{ GN_OP_AnswerCall,            AT_AnswerCall,            Reply },
 	{ GN_OP_CancelCall,            AT_CancelCall,            Reply },
 	{ GN_OP_AT_Ring,               NULL,                     ReplyRing },
 	{ GN_OP_SetCallNotification,   AT_SetCallNotification,   Reply },
 	{ GN_OP_GetNetworkInfo,        AT_GetNetworkInfo,        ReplyGetNetworkInfo },
+	{ GN_OP_SetDateTime,           AT_SetDateTime,           Reply },
+	{ GN_OP_GetDateTime,           AT_GetDateTime,           ReplyGetDateTime },
+	{ GN_OP_SendDTMF,              AT_SendDTMF,              Reply },
+	{ GN_OP_GetActiveCalls,        AT_GetActiveCalls,        ReplyGetActiveCalls },
+	{ GN_OP_OnSMS,                 AT_OnSMS,                 Reply },
+	{ GN_OP_AT_IncomingSMS,        NULL,                     ReplyIncomingSMS },
+	{ GN_OP_AT_GetSMSMemorySize,   AT_GetSMSMemorySize,      ReplyGetSMSMemorySize },
 };
 
 char *strip_quotes(char *s)
@@ -227,7 +252,7 @@ char *memorynames[] = {
 	"CB", /* Currently selected memory */
 };
 
-#define NR_MEMORIES (sizeof (memorynames) / sizeof ((memorynames)[0]))
+#define NR_MEMORIES (sizeof(memorynames) / sizeof((memorynames)[0]))
 
 typedef struct {
 	char *str;
@@ -257,26 +282,28 @@ static gn_error Functions(gn_operation op, gn_data *data, struct gn_statemachine
 }
 
 /* Functions to encode and decode strings */
-int at_encode(int charset, char *dst, char *src, int len)
+size_t at_encode(at_charset charset, char *dst, size_t dst_len, const char *src, size_t len)
 {
+	size_t ret;
+
 	switch (charset) {
 	case AT_CHAR_GSM:
-		len = char_ascii_encode(dst, src, len);
+		ret = char_ascii_encode(dst, dst_len, src, len);
 		break;
 	case AT_CHAR_HEXGSM:
-		len *= 2;
-		char_hex_encode(dst, src, len);
+		ret = char_hex_encode(dst, dst_len, src, len);
 		break;
 	case AT_CHAR_UCS2:
-		len *= 4;
-		char_ucs2_encode(dst, src, len);
+		ret = char_ucs2_encode(dst, dst_len, src, len);
 		break; 
 	default:
-		memcpy(dst, src, len);
+		memcpy(dst, src, dst_len >= len ? len : dst_len);
+		ret = len;
 		break;
 	}
-	dst[len] = '\0';
-	return len;
+	if (ret < dst_len)
+		dst[ret] = '\0';
+	return ret+1;
 }
 
 void at_decode(int charset, char *dst, char *src, int len)
@@ -540,14 +567,13 @@ gn_error at_memory_type_set(gn_memory_type mt, struct gn_statemachine *state)
  *   <mem2> - memory to which writing and sending operations are made
  *   <mem3> - memory to which received SMs are preferred to be stored
  *
- * To avoid confusing the user by changing <mem3> we set it to the current
- * setting.
+ * According to ETSI TS 127 005 V6.0.1 (2005-01) section 3.2.2, second and
+ * third parameter are optional, so we set only <mem1> to be more compatible.
  */
 gn_error AT_SetSMSMemoryType(gn_memory_type mt, struct gn_statemachine *state)
 {
 	at_driver_instance *drvinst = AT_DRVINST(state);
 	gn_data data;
-	gn_sms_status sms_status;
 	char req[32];
 	gn_error ret = GN_ERR_NONE;
 
@@ -555,17 +581,10 @@ gn_error AT_SetSMSMemoryType(gn_memory_type mt, struct gn_statemachine *state)
 		if (mt >= NR_MEMORIES)
 			return GN_ERR_INVALIDMEMORYTYPE;
 		gn_data_clear(&data);
-		data.sms_status = &sms_status;
-		ret = AT_GetSMSStatus(&data, state);
-		if (ret != GN_ERR_NONE)
-			return ret;
-
-		sprintf(req, "AT+CPMS=\"%s\",\"%s\",\"%s\"\r", memorynames[mt], memorynames[mt],
-			memorynames[data.sms_status->new_message_store]);
-		ret = sm_message_send(23, GN_OP_Init, req, state);
-		if (ret != GN_ERR_NONE)
-			return GN_ERR_NOTREADY;
-		ret = sm_block_no_retry(GN_OP_Init, &data, state);
+		sprintf(req, "AT+CPMS=\"%s\"\r", memorynames[mt]);
+		ret = sm_message_send(13, GN_OP_Init, req, state);
+		if (ret == GN_ERR_NONE)
+			ret = sm_block_no_retry(GN_OP_Init, &data, state);
 		if (ret == GN_ERR_NONE)
 			drvinst->smsmemorytype = mt;
 	}
@@ -812,16 +831,18 @@ static gn_error AT_WritePhonebook(gn_data *data, struct gn_statemachine *state)
 		ret = state->driver.functions(GN_OP_AT_SetCharset, data, state);
 		if (ret)
 			return ret;
-		ofs = sprintf(req, "AT+CPBW=%d,\"%s\",%s,\"",
-			      data->phonebook_entry->location+drvinst->memoryoffset,
-			      data->phonebook_entry->number,
-			      data->phonebook_entry->number[0] == '+' ? "145" : "129");
-		len = strlen(data->phonebook_entry->name);
+		ofs = snprintf(req, sizeof(req), "AT+CPBW=%d,\"%s\",%s,\"",
+			       data->phonebook_entry->location+drvinst->memoryoffset,
+			       data->phonebook_entry->number,
+			       data->phonebook_entry->number[0] == '+' ? "145" : "129");
 		tmp = req + ofs;
-		len = at_encode(drvinst->charset, tmp, data->phonebook_entry->name, len);
-		tmp[len++] = '"'; tmp[len++] = '\r';
+		len = at_encode(drvinst->charset, tmp, sizeof(req - ofs),
+				data->phonebook_entry->name,
+				strlen(data->phonebook_entry->name));
+		tmp[len-1] = '"';
+		tmp[len++] = '\r';
 		len += ofs;
-	} 
+	}
 	if (sm_message_send(len, GN_OP_WritePhonebook, req, state))
 		return GN_ERR_NOTREADY;
 	return sm_block_no_retry(GN_OP_WritePhonebook, data, state);
@@ -1058,6 +1079,7 @@ static gn_error AT_GetSMSCenter(gn_data *data, struct gn_statemachine *state)
 	return sm_block_no_retry(GN_OP_GetSMSCenter, data, state);
 }
 
+#ifdef SECURITY
 static gn_error AT_GetSecurityCodeStatus(gn_data *data, struct gn_statemachine *state)
 {
  	if (sm_message_send(9, GN_OP_GetSecurityCodeStatus, "AT+CPIN?\r", state))
@@ -1077,6 +1099,7 @@ static gn_error AT_EnterSecurityCode(gn_data *data, struct gn_statemachine *stat
 		return GN_ERR_NOTREADY;
 	return sm_block_no_retry(GN_OP_EnterSecurityCode, data, state);
 }
+#endif
 
 static gn_error AT_DialVoice(gn_data *data, struct gn_statemachine *state)
 {
@@ -1117,9 +1140,20 @@ static gn_error AT_SetCallNotification(gn_data *data, struct gn_statemachine *st
 			return GN_ERR_NOTREADY;
 		if ((err = sm_block_no_retry(GN_OP_SetCallNotification, data, state)) != GN_ERR_NONE)
 			return err;
+		if (sm_message_send(10, GN_OP_SetCallNotification, "AT+CLIP=1\r", state))
+			return GN_ERR_NOTREADY;
+		/* Ignore errors when we can't set Call Line Identity, just set that we
+		 * don't handle it */
+		if (sm_block_no_retry(GN_OP_SetCallNotification, data, state) == GN_ERR_NONE)
+			drvinst->clip_supported = 1;
+		if (sm_message_send(10, GN_OP_SetCallNotification, "AT+CLCC=1\r", state))
+			return GN_ERR_NOTREADY;
+		/* Ignore errors when we can't set List Current Calls notifications */
+		sm_block_no_retry(GN_OP_SetCallNotification, data, state);
 	}
 
 	drvinst->call_notification = data->call_notification;
+	drvinst->call_callback_data = data->callback_data;
 
 	return GN_ERR_NONE;
 }
@@ -1148,6 +1182,102 @@ static gn_error AT_GetNetworkInfo(gn_data *data, struct gn_statemachine *state)
 	return sm_block_no_retry(GN_OP_GetNetworkInfo, data, state);
 }
 
+static gn_error AT_SetDateTime(gn_data *data, struct gn_statemachine *state)
+{
+	char req[29];
+
+	gn_timestamp *dt = data->datetime;
+
+	sprintf(req, "AT+CCLK=\"%02d/%02d/%02d,%02d:%02d:%02d\"\r",
+		dt->year % 100, dt->month, dt->day,
+		dt->hour, dt->minute, dt->second);
+
+	if (sm_message_send(28, GN_OP_SetDateTime, req, state))
+		return GN_ERR_NOTREADY;
+	return sm_block_no_retry(GN_OP_SetDateTime, data, state);
+}
+
+static gn_error AT_GetDateTime(gn_data *data, struct gn_statemachine *state)
+{
+	if (sm_message_send(9, GN_OP_GetDateTime, "AT+CCLK?\r", state))
+		return GN_ERR_NOTREADY;
+	return sm_block_no_retry(GN_OP_GetDateTime, data, state);
+}
+
+
+/*
+ * This command allows to send DTMF tones and arbitrary tones.
+ * DTMF is a single ASCII character in the set 0-9, *, #, A-D
+ * FIXME: tone and duration parameters are not yet supported
+ */
+static gn_error AT_SendDTMF(gn_data *data, struct gn_statemachine *state)
+{
+	gn_error error = GN_ERR_NONE;
+	unsigned char req[32];
+	int len, i, dtmf_len;
+
+	if (!data || !data->dtmf_string)
+		return GN_ERR_INTERNALERROR;
+
+	dtmf_len = strlen(data->dtmf_string);
+	if (dtmf_len < 1)
+		return GN_ERR_WRONGDATAFORMAT;
+
+	/* First let's check out if the command is supported by the phone */
+	len = snprintf(req, sizeof(req), "AT+VTS=?\r");
+	if (sm_message_send(len, GN_OP_SendDTMF, req, state))
+		return GN_ERR_NOTREADY;
+	if (sm_block_no_retry(GN_OP_SendDTMF, data, state) != GN_ERR_NONE)	
+		return GN_ERR_NOTSUPPORTED;
+
+	/* Send it char by char */
+	for (i = 0; i < dtmf_len; i++) {
+		len = snprintf(req, sizeof(req), "AT+VTS=%c\r", data->dtmf_string[i]);
+		if (sm_message_send(len, GN_OP_SendDTMF, req, state))
+			return GN_ERR_NOTREADY;
+		error = sm_block_no_retry(GN_OP_SendDTMF, data, state);
+		if (error)
+			break;
+	}
+
+	return error;
+}
+
+static gn_error AT_GetActiveCalls(gn_data *data, struct gn_statemachine *state)
+{
+	if (sm_message_send(8, GN_OP_GetActiveCalls, "AT+CPAS\r", state))
+		return GN_ERR_NOTREADY;
+	return sm_block_no_retry(GN_OP_GetActiveCalls, data, state);
+}
+
+static gn_error AT_OnSMS(gn_data *data, struct gn_statemachine *state)
+{
+	gn_error error;
+	int i = 2;
+	char req[13];
+
+	do {
+		snprintf(req, 13, "AT+CNMI=%d,1\r", i);
+		if (sm_message_send(12, GN_OP_OnSMS, req, state))
+			return GN_ERR_NOTREADY;
+
+		error = sm_block_no_retry(GN_OP_OnSMS, data, state);
+	} while (i-- && error);
+	if (error == GN_ERR_NONE) {
+		AT_DRVINST(state)->on_sms = data->on_sms;
+		AT_DRVINST(state)->sms_callback_data = data->callback_data;
+	}
+	return error;
+}
+
+static gn_error AT_GetSMSMemorySize(gn_data *data, struct gn_statemachine *state)
+{
+	if (sm_message_send(18, GN_OP_AT_GetSMSMemorySize, "AT+CPMS=\"ME\",\"SM\"\r", state))
+		return GN_ERR_NOTREADY;
+
+	return sm_block_no_retry(GN_OP_AT_GetSMSMemorySize, data, state);
+}
+
 static gn_error ReplyReadPhonebook(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
 	at_driver_instance *drvinst = AT_DRVINST(state);
@@ -1171,14 +1301,14 @@ static gn_error ReplyReadPhonebook(int messagetype, unsigned char *buffer, int l
 		if (data->phonebook_entry) {
 			*(data->phonebook_entry->number) = '\0';
 			*(data->phonebook_entry->name) = '\0';
-			data->phonebook_entry->caller_group = 0;
+			data->phonebook_entry->caller_group = GN_PHONEBOOK_GROUP_None;
 			data->phonebook_entry->subentries_count = 0;
 			data->phonebook_entry->empty = true;
 		}
 		return GN_ERR_NONE;
 	}
 	if (data->phonebook_entry) {
-		data->phonebook_entry->caller_group = 0;
+		data->phonebook_entry->caller_group = GN_PHONEBOOK_GROUP_None;
 		data->phonebook_entry->subentries_count = 0;
 		data->phonebook_entry->empty = false;
 
@@ -1333,9 +1463,10 @@ static gn_error ReplyGetBattery(int messagetype, unsigned char *buffer, int leng
 	
 	splitlines(&buf);
 
-	if (!strncmp(buf.line1, "AT+CBC", 6)) { /* FIXME realy needed? */
+	if (!strncmp(buf.line1, "AT+CBC", 6) && !strncmp(buf.line2, "+CBC: ", 6)) {
 		if (data->battery_level) {
-			*(data->battery_unit) = GN_BU_Percentage;
+			if (data->battery_unit)
+				*(data->battery_unit) = GN_BU_Percentage;
 			pos = strchr(buf.line2, ',');
 			if (pos) {
 				pos++;
@@ -1346,8 +1477,11 @@ static gn_error ReplyGetBattery(int messagetype, unsigned char *buffer, int leng
 		}
 		if (data->power_source) {
 			*(data->power_source) = 0;
-			if (*buf.line2 == '1') *(data->power_source) = GN_PS_ACDC;
-			if (*buf.line2 == '0') *(data->power_source) = GN_PS_BATTERY;
+			pos = buf.line2 + strlen("+CBC: ");
+			if (*pos == '1' ||  *pos == '2')
+				*(data->power_source) = GN_PS_ACDC;
+			else if (*pos == '0')
+				*(data->power_source) = GN_PS_BATTERY;
 		}
 	}
 	return GN_ERR_NONE;
@@ -1384,7 +1518,8 @@ static gn_error ReplyIdentify(int messagetype, unsigned char *buffer, int length
 	at_line_buffer buf;
 	gn_error error;
 
-	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
+		return error;
 
 	buf.line1 = buffer + 1;
 	buf.length = length;
@@ -1430,8 +1565,6 @@ static gn_error ReplyGetSMSStatus(int messagetype, unsigned char *buffer, int le
 {
 	at_line_buffer buf;
 	gn_error error;
-	char store[3] = "XX";
-	int i;
 
 	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
 
@@ -1439,22 +1572,13 @@ static gn_error ReplyGetSMSStatus(int messagetype, unsigned char *buffer, int le
 	buf.length = length;
 	splitlines(&buf);
 
-	if (sscanf(buf.line2, "+CPMS: \"%*c%*c\",%d,%*d,\"%*c%*c\",%*d,%*d,\"%c%c\",%*d,%*d",
-		   &data->sms_status->number, &store[0], &store[1]) != 3)
+	if (sscanf(buf.line2, "+CPMS: \"%*c%*c\",%d,%*d", &data->sms_status->number) != 1)
 		return GN_ERR_FAILED;
 
 	data->sms_status->unread = 0;
 	data->sms_status->changed = 0;
 	data->sms_status->folders_count = 0;
 
-	data->sms_status->new_message_store = GN_MT_ME;
-	for (i = 0; i < NR_MEMORIES; i++) {
-		if (strcmp(store, memorynames[i]) == 0) {
-			data->sms_status->new_message_store = i;
-			break;
-		}
-	}
-
 	return GN_ERR_NONE;
 }
 
@@ -1484,20 +1608,76 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 	at_line_buffer buf;
 	gn_error ret = GN_ERR_NONE;
 	unsigned int sms_len, l, offset = 0;
-	char *tmp;
+	unsigned char *tmp;
 	gn_error error;
  	at_driver_instance *drvinst = AT_DRVINST(state);
 
-	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE) return error;
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
+		return error;
 	
 	buf.line1 = buffer + 1;
 	buf.length = length;
 
 	splitlines(&buf);
 
-	if (!data->raw_sms) return GN_ERR_INTERNALERROR;
+	if (!data->raw_sms)
+		return GN_ERR_INTERNALERROR;
+
+	/* Try to figure out the status first */
+	tmp = strchr(buf.line2, ',');
+	if (tmp != NULL && ((char *) tmp - buf.line2 - strlen("+CMGR: ")) >= 1) {
+		char *status;
+		int len;
+
+		len = (char *) tmp - buf.line2 - strlen("+CMGR: ");
+		status = malloc(len + 1);
+		if (!status) {
+			dprintf("Not enough memory for buffer.\n");
+			return GN_ERR_INTERNALERROR;
+		}
+
+		memcpy(status, buf.line2 + strlen("+CMGR: "), len);
+		status[len] = '\0';
+
+		if (strstr(status, "UNREAD")) {
+			data->raw_sms->status = GN_SMS_Unread;
+		} else if (strstr(status, "READ")) {
+			data->raw_sms->status = GN_SMS_Read;
+		} else if (strstr(status, "UNSENT")) {
+			data->raw_sms->status = GN_SMS_Unsent;
+		} else if (strstr(status, "SENT")) {
+			data->raw_sms->status = GN_SMS_Sent;
+		} else {
+			int s;
+
+			s = atoi(status);
+			switch (s) {
+			case 0:
+				data->raw_sms->status = GN_SMS_Unread;
+				break;
+			case 1:
+				data->raw_sms->status = GN_SMS_Read;
+				break;
+			case 2:
+				data->raw_sms->status = GN_SMS_Unsent;
+				break;
+			case 3:
+				data->raw_sms->status = GN_SMS_Sent;
+				break;
+			}
+		}
+		free(status);
+	}
 
 	tmp = strrchr(buf.line2, ',');
+	/* The following sequence is correct for emtpy location:
+	 * w: AT+CMGR=9
+	 * r: AT+CMGR=9
+	 *  :
+	 *  : OK
+	 */
+	if (!tmp)
+		return GN_ERR_EMPTYLOCATION;
 	sms_len = atoi(tmp+1);
 	if (sms_len == 0)
 		return GN_ERR_EMPTYLOCATION;
@@ -1522,33 +1702,76 @@ static gn_error ReplyGetSMS(int messagetype, unsigned char *buffer, int length,
 		offset += l;
 	}
 
+	data->raw_sms->reject_duplicates   = 0;
+	data->raw_sms->report_status       = 0;
+	data->raw_sms->reference           = 0;
+	data->raw_sms->reply_via_same_smsc = 0;
+	data->raw_sms->report              = 0;
+
 	data->raw_sms->type                = (tmp[offset] & 0x03) << 1;
-	data->raw_sms->udh_indicator       = tmp[offset];
-	data->raw_sms->more_messages       = tmp[offset];
-	data->raw_sms->report_status       = tmp[offset];
-	l = (tmp[offset + 1] % 2) ? tmp[offset + 1] + 1 : tmp[offset + 1] ;
+	if (data->raw_sms->type == GN_SMS_MT_Deliver) { 
+		dprintf("SMS-DELIVER found\n");
+		data->raw_sms->more_messages       = tmp[offset] & 0x04;
+		// bits 3 and 4 of the first octet unused;
+		data->raw_sms->report_status       = tmp[offset] & 0x20;
+	} else if (data->raw_sms->type == GN_SMS_MT_Submit) {
+		dprintf("SMS-SUBMIT found\n");
+		data->raw_sms->reject_duplicates   = tmp[offset] & 0x04;
+		data->raw_sms->validity_indicator  = (tmp[offset] & 0x18) >> 3;
+	} else {
+		dprintf("Unknown PDU type %d\n", data->raw_sms->type);
+		return GN_ERR_INTERNALERROR;
+ 	}
+ 	data->raw_sms->more_messages       = tmp[offset];
+	data->raw_sms->udh_indicator       = tmp[offset] & 0x40;
+	offset++;
+
+	if (data->raw_sms->type == GN_SMS_MT_Submit) {
+		data->raw_sms->reference = tmp[offset++];
+	} else {
+		data->raw_sms->reference = 0;
+	}
+
+	l = (tmp[offset] % 2) ? tmp[offset] + 1 : tmp[offset];
 	l = l / 2 + 2;
-	if (l + offset + 11 > sms_len || l > GN_SMS_NUMBER_MAX_LENGTH) {
+	if (l + offset + 10 > sms_len || l > GN_SMS_NUMBER_MAX_LENGTH) {
 		dprintf("Invalid remote number length (%d)\n", l);
 		ret = GN_ERR_INTERNALERROR;
 		goto out;
 	}
-	memcpy(data->raw_sms->remote_number, tmp + offset + 1, l);
+	memcpy(data->raw_sms->remote_number, tmp + offset, l);
 	offset += l;
-	data->raw_sms->reply_via_same_smsc = 0;
-	data->raw_sms->reject_duplicates   = 0;
-	data->raw_sms->report              = 0;
-	data->raw_sms->reference           = 0;
-	data->raw_sms->pid                 = tmp[offset + 1];
-	data->raw_sms->dcs                 = tmp[offset + 2];
-	memcpy(data->raw_sms->smsc_time, tmp + offset + 3, 7);
-	data->raw_sms->length              = tmp[offset + 10] & 0x00ff;
-	if (sms_len - offset - 11 > 1000) {
+	data->raw_sms->pid                 = tmp[offset++];
+	data->raw_sms->dcs                 = tmp[offset++];
+	if (data->raw_sms->type == GN_SMS_MT_Deliver) {
+		memcpy(data->raw_sms->smsc_time, tmp + offset, 7);
+		offset += 7;
+	}
+	if (data->raw_sms->type == GN_SMS_MT_Submit) {
+		if (data->raw_sms->validity_indicator == GN_SMS_VP_None) {
+			offset += 0;
+		} else if (data->raw_sms->validity_indicator == GN_SMS_VP_RelativeFormat) {
+			// FIXME save this info
+			offset += 1;
+		} else if (data->raw_sms->validity_indicator == GN_SMS_VP_EnhancedFormat ||
+			   data->raw_sms->validity_indicator == GN_SMS_VP_AbsoluteFormat) {
+			// FIXME save this info
+			offset += 7;
+		} else { 
+			dprintf("Internal Error on validity_indicator");
+			return GN_ERR_INTERNALERROR;
+		}
+	}
+	data->raw_sms->length              = tmp[offset++];
+	data->raw_sms->user_data_length = data->raw_sms->length;
+	if (data->raw_sms->udh_indicator & 0x40)
+		data->raw_sms->user_data_length -= tmp[offset] + 1;
+	if (sms_len - offset > 1000) {
 		dprintf("Phone gave as poisonous (too short?) reply %s, either phone went crazy or communication went out of sync\n", buf.line3);
 		ret = GN_ERR_INTERNALERROR;
 		goto out;
 	}
-	memcpy(data->raw_sms->user_data, tmp + offset + 11, sms_len - offset - 11);
+	memcpy(data->raw_sms->user_data, tmp + offset, sms_len - offset);
 out:
 	free(tmp);
 	return ret;
@@ -1595,6 +1818,7 @@ static gn_error ReplyGetCharset(int messagetype, unsigned char *buffer, int leng
 	return GN_ERR_FAILED;
 }
 
+#ifdef SECURITY
 static gn_error ReplyGetSecurityCodeStatus(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
 	at_line_buffer buf;
@@ -1638,6 +1862,7 @@ static gn_error ReplyGetSecurityCodeStatus(int messagetype, unsigned char *buffe
 	}
 	return GN_ERR_NONE;
 }
+#endif
 
 static gn_error ReplyRing(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
@@ -1645,6 +1870,7 @@ static gn_error ReplyRing(int messagetype, unsigned char *buffer, int length, gn
 	at_line_buffer buf;
 	char *pos;
 	gn_call_info cinfo;
+	gn_call_status status;
 
 	if (!drvinst->call_notification) return GN_ERR_UNSOLICITED;
 
@@ -1655,39 +1881,230 @@ static gn_error ReplyRing(int messagetype, unsigned char *buffer, int length, gn
 	memset(&cinfo, 0, sizeof(cinfo));
 	cinfo.call_id = 1;
 
-	if (!strncmp(buf.line1, "RING", 4))
+	if (!strncmp(buf.line1, "RING", 4)) {
 		return GN_ERR_INTERNALERROR; /* AT+CRC=1 disables RING */
-
-	else if (!strncmp(buf.line1, "+CRING: ", 8)) {
+	} else if (!strncmp(buf.line1, "+CRING: ", 8)) {
 		pos = buf.line1 + 8;
 		if (!strncmp(pos, "VOICE", 5))
 			cinfo.type = GN_CALL_Voice;
 		else
 			return GN_ERR_UNHANDLEDFRAME;
+		status = GN_CALL_Incoming;
+	} else if (!strncmp(buf.line1, "CONNECT", 7)) {
+		status = GN_CALL_Established;
+	} else if (!strncmp(buf.line1, "BUSY", 4)) {
+		status = GN_CALL_RemoteHangup;
+	} else if (!strncmp(buf.line1, "NO ANSWER", 9)) {
+		status = GN_CALL_RemoteHangup;
+	} else if (!strncmp(buf.line1, "NO CARRIER", 10)) {
+		status = GN_CALL_RemoteHangup;
+	} else if (!strncmp(buf.line1, "NO DIALTONE", 11)) {
+		status = GN_CALL_LocalHangup;
+	} else if (!strncmp(buf.line1, "+CLIP: ", 7)) {
+		char **items;
+		int i;
 
-		drvinst->call_notification(GN_CALL_Incoming, &cinfo, state);
+		items = gnokii_strsplit(buf.line1 + 7, ",", 6);
+		for (i = 0; i < 6; i++) {
+			if (items[i] == NULL)
+				break;
+			switch (i) {
+			/* Number, if known */
+			case 0:
+				snprintf(cinfo.number, GN_PHONEBOOK_NUMBER_MAX_LENGTH, "%s", strip_quotes(items[i]));
+				break;
+			/* 1 == Number "type"
+			 * 2 == String type subaddress
+			 * 3 == Type of subaddress
+			 * All ignored
+			 */
+			case 1:
+			case 2:
+			case 3:
+				break;
+			/* Name, if known */
+			case 4:
+				snprintf(cinfo.name, GN_PHONEBOOK_NAME_MAX_LENGTH, "%s", strip_quotes(items[i]));
+				break;
+			/* Validity of the name/number provided */
+			case 5:
+				switch (atoi(items[i])) {
+				case 1:
+					snprintf(cinfo.name, GN_PHONEBOOK_NAME_MAX_LENGTH, _("Withheld"));
+					break;
+				case 2:
+					snprintf(cinfo.name, GN_PHONEBOOK_NAME_MAX_LENGTH, _("Unknown"));
+					break;
+				}
+			}
+		}
 
-	} else if (!strncmp(buf.line1, "CONNECT", 7))
-		drvinst->call_notification(GN_CALL_Established, &cinfo, state);
+		if (cinfo.name == NULL)
+			snprintf(cinfo.name, GN_PHONEBOOK_NAME_MAX_LENGTH, _("Unknown"));
+		cinfo.type = drvinst->last_call_type;
+		drvinst->call_notification(drvinst->last_call_status, &cinfo, state, drvinst->call_callback_data);
+		gnokii_strfreev(items);
+		return GN_ERR_UNSOLICITED;
+	} else if (!strncmp(buf.line1, "+CLCC: ", 7)) {
+		char **items;
+		int i;
 
-	else if (!strncmp(buf.line1, "BUSY", 4))
-		drvinst->call_notification(GN_CALL_RemoteHangup, &cinfo, state);
+		items = gnokii_strsplit(buf.line1 + 7, ",", 8);
+		status = -1;
 
-	else if (!strncmp(buf.line1, "NO ANSWER", 9))
-		drvinst->call_notification(GN_CALL_RemoteHangup, &cinfo, state);
+		for (i = 0; i < 8; i++) {
+			if (items[i] == NULL)
+				break;
+			switch (i) {
+			/* Call ID */
+			case 0:
+				cinfo.call_id = atoi(items[i]);
+				break;
+			/* Incoming, or finishing call */
+			case 1:
+				break;
+			case 2:
+				switch (atoi(items[i])) {
+				case 0:
+					status = GN_CALL_Established;
+					break;
+				case 1:
+					status = GN_CALL_Held;
+					break;
+				case 2:
+					status = GN_CALL_Dialing;
+					break;
+				case 3:
+					/* Alerting, remote end is ringing */
+					status = GN_CALL_Ringing;
+					break;
+				case 4:
+					status = GN_CALL_Incoming;
+					break;
+				case 5:
+					/* FIXME Call is "Waiting" */
+					status = GN_CALL_Held;
+					break;
+				case 6:
+					status = GN_CALL_LocalHangup;
+					break;
+				}
+				break;
+			case 3:
+				if (atoi(items[i]) == 0)
+					cinfo.type = GN_CALL_Voice;
+				else {
+					/* We don't handle non-voice calls */
+					gnokii_strfreev(items);
+					return GN_ERR_UNHANDLEDFRAME;
+				}
+				break;
+			/* Multiparty status */
+			case 4:
+				break;
+			case 5:
+				snprintf(cinfo.number, GN_PHONEBOOK_NUMBER_MAX_LENGTH, "%s", strip_quotes (items[i]));
+				break;
+			/* Phone display format */
+			case 6:
+				break;
+			case 7:
+				snprintf(cinfo.name, GN_PHONEBOOK_NAME_MAX_LENGTH, "%s", strip_quotes (items[i]));
+				break;
+			}
+		}
 
-	else if (!strncmp(buf.line1, "NO CARRIER", 10))
-		drvinst->call_notification(GN_CALL_RemoteHangup, &cinfo, state);
+		/* FIXME we don't handle calls > 1 anywhere else */
+		if (status < 0) {
+			gnokii_strfreev(items);
+			return GN_ERR_UNHANDLEDFRAME;
+		}
 
-	else if (!strncmp(buf.line1, "NO DIALTONE", 11))
-		drvinst->call_notification(GN_CALL_LocalHangup, &cinfo, state);
-
-	else
+		drvinst->call_notification(status, &cinfo, state, drvinst->call_callback_data);
+		gnokii_strfreev(items);
+		return GN_ERR_UNSOLICITED;
+	} else {
 		return GN_ERR_UNHANDLEDFRAME;
+	}
+
+	if (!drvinst->clip_supported || status != GN_CALL_Incoming) {
+		drvinst->call_notification(status, &cinfo, state, drvinst->call_callback_data);
+	} else {
+		drvinst->last_call_status = status;
+		drvinst->last_call_type = cinfo.type;
+	}
 
 	return GN_ERR_UNSOLICITED;
 }
 
+static gn_error ReplyIncomingSMS(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
+{
+	at_driver_instance *drvinst = AT_DRVINST(state);
+	at_line_buffer buf;
+	char *memory, *pos;
+	int index, i;
+	gn_memory_type mem;
+	int freesms = 0;
+	gn_error error = GN_ERR_NONE;
+
+	if (!drvinst->on_sms)
+		return GN_ERR_UNSOLICITED;
+
+	buf.line1 = buffer;
+	buf.length= length;
+	splitlines(&buf);
+
+	mem = GN_MT_XX;
+
+	if (strncmp(buf.line1, "+CMTI: ", 7))
+		return GN_ERR_UNSOLICITED;
+
+	pos = strrchr(buf.line1, ',');
+	if (pos == NULL)
+		return GN_ERR_UNSOLICITED;
+	pos[0] = '\0';
+	pos++;
+	index = atoi(pos);
+
+	memory = strip_quotes(buf.line1 + 7);
+	if (memory == NULL)
+		return GN_ERR_UNSOLICITED;
+	for (i = 0; i < NR_MEMORIES; i++) {
+		if (!strcmp(memory, memorynames[i])) {
+			mem = i;
+			break;
+		}
+	}
+
+	if (mem == GN_MT_XX)
+		return GN_ERR_UNSOLICITED;
+
+	dprintf("Received message folder %s index %d\n", memorynames[mem], index);
+
+	if (!data->sms) {
+		freesms = 1;
+		data->sms = calloc(1, sizeof(gn_sms));
+		if (!data->sms)
+			return GN_ERR_INTERNALERROR;
+	}
+
+	memset(data->sms, 0, sizeof(gn_sms));
+	data->sms->memory_type = mem;
+	data->sms->number = index;
+
+	dprintf("get sms %d\n", index);
+	error = gn_sms_get(data, state);
+	if (error == GN_ERR_NONE) {
+		error = GN_ERR_UNSOLICITED;
+		drvinst->on_sms(data->sms, state, drvinst->sms_callback_data);
+	}
+
+	if (freesms)
+		free(data->sms);
+
+	return error;
+}
+
 static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
 {
 	at_driver_instance *drvinst = AT_DRVINST(state);
@@ -1790,6 +2207,123 @@ static gn_error ReplyGetNetworkInfo(int messagetype, unsigned char *buffer, int
 	return GN_ERR_NONE;
 }
 
+static gn_error ReplyGetDateTime(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
+{
+	at_line_buffer buf;
+	gn_error error;
+	gn_timestamp *dt;
+
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
+		return error;
+
+	buf.line1 = buffer + 1;
+	buf.length= length;
+
+	splitlines(&buf);
+
+	dt = data->datetime;
+	if (sscanf(buf.line2, "+CCLK: \"%d/%d/%d,%d:%d:%d\"",
+		   &dt->year, &dt->month, &dt->day,
+		   &dt->hour, &dt->minute, &dt->second) != 6)
+		return GN_ERR_FAILED;
+
+	if (dt->year < 100)
+		dt->year += 2000;
+
+	return GN_ERR_NONE;
+}
+
+static gn_error ReplyGetActiveCalls(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
+{
+	at_driver_instance *drvinst;
+	gn_error error;
+	at_line_buffer buf;
+	int status;
+
+	if (!data->call_active)
+		return GN_ERR_INTERNALERROR;
+
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
+		return error;
+
+	status = -1;
+
+	buf.line1 = buffer + 1;
+	buf.length = length;
+	splitlines(&buf);
+
+	memset(data->call_active, 0, GN_CALL_MAX_PARALLEL * sizeof(gn_call_active));
+
+	if (strncmp(buf.line1, "AT+CPAS", 7)) {
+		return GN_ERR_UNKNOWN;
+	}
+
+	data->call_active->call_id = 1;
+
+	switch (atoi(buf.line2 + strlen("+CPAS: "))) {
+	case 0:
+		status = GN_CALL_Idle;
+		break;
+	/* Terminal doesn't know */
+	case 1:
+	case 2:
+		break;
+	case 3:
+		status = GN_CALL_Ringing;
+		break;
+	case 4:
+		status = GN_CALL_Established;
+		break;
+	/* Low-power state, terminal can't answer */
+	case 5:
+		break;
+	}
+
+	if (status < 0)
+		return GN_ERR_UNKNOWN;
+
+	drvinst = AT_DRVINST(state);
+
+	data->call_active->state = status;
+	data->call_active->prev_state = drvinst->prev_state;
+
+	/* States go:
+	 * Idle -> Ringing -> (faked Local hangup) -> Idle
+	 * Idle -> Ringing -> Established -> (faked Remote hangup) -> Idle */
+	if (drvinst->prev_state == GN_CALL_Ringing && status == GN_CALL_Idle)
+		data->call_active->state = GN_CALL_LocalHangup;
+	else if (drvinst->prev_state == GN_CALL_Established && status == GN_CALL_Idle)
+		data->call_active->state = GN_CALL_RemoteHangup;
+	else
+		data->call_active->state = status;
+
+	drvinst->prev_state = data->call_active->state;
+	snprintf(data->call_active->name, GN_PHONEBOOK_NAME_MAX_LENGTH, _("Unknown"));
+	data->call_active->number[0] = '\0';
+
+	return GN_ERR_NONE;
+}
+
+static gn_error ReplyGetSMSMemorySize(int messagetype, unsigned char *buffer, int length, gn_data *data, struct gn_statemachine *state)
+{
+	at_driver_instance *drvinst = AT_DRVINST(state);
+	at_line_buffer buf;
+	gn_error error;
+
+	if ((error = at_error_get(buffer, state)) != GN_ERR_NONE)
+		return error;
+
+	buf.line1 = buffer + 1;
+	buf.length = length;
+	splitlines(&buf);
+
+	if (sscanf(buf.line2, "+CPMS: %*d,%d,%*d,%d", &drvinst->mememorysize, &drvinst->smmemorysize) != 2)
+		return GN_ERR_FAILED;
+	drvinst->smsmemorytype = GN_MT_ME;
+
+	return GN_ERR_NONE;
+}
+
 /* General reply function for phone responses. buffer[0] holds the compiled
  * success of the result (OK, ERROR, ... ). see links/atbus.h and links/atbus.c 
  * for reference */
@@ -1808,7 +2342,11 @@ static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 	int i;
 
 	dprintf("Initializing AT capable mobile phone ...\n");
-	
+
+	/* initialize variables */
+	memset(model, 0, GN_MODEL_MAX_LENGTH);
+	memset(manufacturer, 0, GN_MANUFACTURER_MAX_LENGTH);
+
 	/* Copy in the phone info */
 	memcpy(&(state->driver), &driver_at, sizeof(gn_driver));
 
@@ -1826,6 +2364,17 @@ static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 	drvinst->charset = AT_CHAR_UNKNOWN;
 	drvinst->no_smsc = 0;
 	drvinst->call_notification = NULL;
+	drvinst->call_callback_data = NULL;
+	drvinst->on_cell_broadcast = NULL;
+	drvinst->cb_callback_data = NULL;
+	drvinst->on_sms = NULL;
+	drvinst->sms_callback_data = NULL;
+	drvinst->clip_supported = 0;
+	drvinst->last_call_type = GN_CALL_Voice;
+	drvinst->last_call_status = GN_CALL_Idle;
+	drvinst->prev_state = GN_CALL_Idle;
+	drvinst->mememorysize = -1;
+	drvinst->smmemorysize = -1;
 
 	drvinst->if_pos = 0;
 	for (i = 0; i < GN_OP_AT_Max; i++) {
@@ -1884,6 +2433,8 @@ static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 		at_siemens_init(model, setupdata->model, state);
 	else if (!strncasecmp(manufacturer, "sony ericsson", 14))
 		at_sonyericsson_init(model, setupdata->model, state);
+	else if (!strncasecmp(manufacturer, "samsung", 7))
+		at_samsung_init(model, setupdata->model, state);
 	
 	StoreDefaultCharset(state);
 
@@ -1891,8 +2442,8 @@ static gn_error Initialise(gn_data *setupdata, struct gn_statemachine *state)
 out:
 	if (ret) {
 		dprintf("Initialization failed (%d)\n", ret);
-		free(AT_DRVINST(state));
-		AT_DRVINST(state) = NULL;
+		/* ignore return value from GN_OP_Terminate, will use previous error code instead */
+		state->driver.functions(GN_OP_Terminate, &data, state);
 	}
 	return ret;
 }
@@ -1909,29 +2460,33 @@ static gn_error Terminate(gn_data *data, struct gn_statemachine *state)
 void splitlines(at_line_buffer *buf)
 {
 	char *pos;
+	int length = buf->length;
 
-	pos = findcrlf(buf->line1, 0, buf->length);
+	pos = findcrlf(buf->line1, 0, length);
 	if (pos) {
 		*pos = 0;
 		buf->line2 = skipcrlf(++pos);
+		length -= (buf->line2 - buf->line1);
 	} else {
 		buf->line2 = buf->line1;
 	}
-	pos = findcrlf(buf->line2, 1, buf->length);
+	pos = findcrlf(buf->line2, 1, length);
 	if (pos) {
 		*pos = 0;
 		buf->line3 = skipcrlf(++pos);
+		length -= (buf->line3 - buf->line2);
 	} else {
 		buf->line3 = buf->line2;
 	}
-	pos = findcrlf(buf->line3, 1, buf->length);
+	pos = findcrlf(buf->line3, 1, length);
 	if (pos) {
 		*pos = 0;
 		buf->line4 = skipcrlf(++pos);
+		length -= (buf->line4 - buf->line3);
 	} else {
 		buf->line4 = buf->line3;
 	}
-	pos = findcrlf(buf->line4, 1, buf->length);
+	pos = findcrlf(buf->line4, 1, length);
 	if (pos) {
 		*pos = 0;
 	}
@@ -1961,7 +2516,7 @@ char *findcrlf(unsigned char *str, int test, int max)
 {
 	if (str == NULL)
 		return str;
-	while ((*str != '\n') && (*str != '\r') && ((*str != '\0') || test) && (max > 0)) {
+	while ((max > 0) && (*str != '\n') && (*str != '\r') && ((*str != '\0') || test)) {
 		str++;
 		max--;
 	}
