@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnapplet.c,v 1.30 2006/01/05 00:25:30 bozo Exp $
+  $Id: gnapplet.c,v 1.33 2006/10/03 21:26:37 pkot Exp $
 
   G N O K I I
 
@@ -35,10 +35,7 @@
 #include <ctype.h>
 #include <assert.h>
 
-#ifdef HAVE_UNISTD_H
-#  include <unistd.h>
-#endif
-
+#include "compat.h"
 #include "misc.h"
 #include "phones/generic.h"
 #include "phones/gnapplet.h"
@@ -147,7 +144,7 @@ gn_driver driver_gnapplet = {
 	pgen_incoming_default,
 	/* Mobile phone information */
 	{
-		"gnapplet|series60|3650|6600|sx1|6680",	/* Supported models */
+		"gnapplet|symbian|3650|6600|sx1",	/* Supported models */
 		0,			/* Max RF Level */
 		100,			/* Min RF Level */
 		GN_RF_Percentage,	/* RF level units */
@@ -570,7 +567,7 @@ static gn_error gnapplet_identify(gn_data *data, struct gn_statemachine *state)
 			return error;
 	}
 
-	if (data->manufacturer) snprintf(data->manufacturer, 20, "%s", drvinst->manufacturer);
+	if (data->manufacturer) snprintf(data->manufacturer, GN_MANUFACTURER_MAX_LENGTH, "%s", drvinst->manufacturer);
 	if (data->model) snprintf(data->model, GN_MODEL_MAX_LENGTH, "%s", drvinst->model);
 	if (data->imei) snprintf(data->imei, GN_IMEI_MAX_LENGTH, "%s", drvinst->imei);
 	if (data->revision) snprintf(data->revision, GN_REVISION_MAX_LENGTH, "SW %s, HW %s", drvinst->sw_version, drvinst->hw_version);
