@@ -1,6 +1,6 @@
 /*
 
-  $Id: vcal.c,v 1.12 2006/06/11 16:39:48 deller Exp $
+  $Id: vcal.c,v 1.18 2007/07/09 18:28:59 pkot Exp $
 
   G N O K I I
 
@@ -42,6 +42,8 @@ static const char gnokii_version[] = VERSION;
 
 
 #ifdef HAVE_LIBICAL
+/* needs gnokii-internal.h for string_base64() and utf8_encode() */
+#  include "gnokii-internal.h"
 #  include "ical.h"
 /* mother's little helpers */
 static int iterate_cal(icalcomponent *parent, int depth, int *count, icalcomponent **result, icalcomponent_kind kind);
@@ -195,7 +197,7 @@ GNOKII_API int gn_calnote2ical(FILE *f, gn_calnote *calnote)
 		icalcomponent_free(pIcal);
 		pIcal = NULL;
 	} else {
-		fprintf(stderr, _("ERROR\n"));
+		dprintf("ERROR in icalcomponent_vanew()\n");
 	}
 	return GN_ERR_NONE;
 #else
@@ -248,22 +250,34 @@ GNOKII_API int gn_calnote2ical(FILE *f, gn_calnote *calnote)
 	case GN_CALNOTE_NEVER: 
 		break; 
 	case GN_CALNOTE_DAILY: 
-		fprintf(f, "RRULE:FREQ=DAILY\r\n"); 
+		calnote->occurrences ?
+			fprintf(f, "RRULE:FREQ=DAILY\r\n") :
+			fprintf(f, "RRULE:FREQ=DAILY;COUNT=%d\r\n", calnote->occurrences);
 		break; 
 	case GN_CALNOTE_WEEKLY: 
-		fprintf(f, "RRULE:FREQ=WEEKLY\r\n"); 
+		calnote->occurrences ?
+			fprintf(f, "RRULE:FREQ=WEEKLY\r\n") :
+			fprintf(f, "RRULE:FREQ=WEEKLY;COUNT=%d\r\n", calnote->occurrences);
 		break; 
 	case GN_CALNOTE_2WEEKLY: 
-		fprintf(f, "RRULE:FREQ=WEEKLY;INTERVAL=2\r\n"); 
+		calnote->occurrences ?
+			fprintf(f, "RRULE:FREQ=WEEKLY;INTERVAL=2\r\n") :
+			fprintf(f, "RRULE:FREQ=WEEKLY;INTERVAL=2;COUNT=%d\r\n", calnote->occurrences);
 		break; 
 	case GN_CALNOTE_MONTHLY: 
-		fprintf(f, "RRULE:FREQ=MONTHLY\r\n"); 
+		calnote->occurrences ?
+			fprintf(f, "RRULE:FREQ=MONTHLY\r\n") :
+			fprintf(f, "RRULE:FREQ=MONTHLY;COUNT=%d\r\n", calnote->occurrences);
 		break; 
 	case GN_CALNOTE_YEARLY: 
-		fprintf(f, "RRULE:FREQ=YEARLY\r\n"); 
+		calnote->occurrences ?
+			fprintf(f, "RRULE:FREQ=YEARLY\r\n") :
+			fprintf(f, "RRULE:FREQ=YEARLY;COUNT=%d\r\n", calnote->occurrences);
 		break; 
 	default: 
-		fprintf(f, "RRULE:FREQ=HOURLY;INTERVAL=%d\r\n", calnote->recurrence); 
+		calnote->occurrences ?
+			fprintf(f, "RRULE:FREQ=HOURLY;INTERVAL=%d\r\n", calnote->recurrence) :
+			fprintf(f, "RRULE:FREQ=HOURLY;INTERVAL=%d;COUNT=%d\r\n", calnote->recurrence, calnote->occurrences);
 		break; 
 	} 
 	fprintf(f, "END:VEVENT\r\n"); 
@@ -282,7 +296,7 @@ static inline gn_calnote_type str2calnote_type(const char *str)
 		return GN_CALNOTE_MEETING;
 	} else if (!strncasecmp("ANNIVERSARY", str, 11)) {
 		return GN_CALNOTE_BIRTHDAY;
-	} else if (!strncasecmp("MISCELLANOUS", str, 12)) {
+	} else if (!strncasecmp("MISCELLANEOUS", str, 13)) {
 		return GN_CALNOTE_MEMO;
 	}
 	return GN_CALNOTE_REMINDER;
@@ -313,11 +327,12 @@ GNOKII_API int gn_ical2calnote(FILE *f, gn_calnote *calnote, int id)
 	if (id < 1)
 		id = 1;
 
-	/* interate through the component */
+	/* iterate through the component */
 	iterate_cal(comp, 0, &id, &compresult, ICAL_VEVENT_COMPONENT);
 
 	if (!compresult) {
-		fprintf(stderr, _("No component found.\n"));
+		dprintf("No component found.\n");
+		retval = GN_ERR_EMPTYLOCATION;
 	} else {
 		const char *str;
 		icalcomponent *alarm = {0};
@@ -413,7 +428,7 @@ GNOKII_API int gn_ical2calnote(FILE *f, gn_calnote *calnote, int id)
 		if (!str) str = "";
 		snprintf(calnote->mlocation, sizeof(calnote->mlocation), "%s", str);
 
-		fprintf(stderr, _("Component found\n%s\n"), icalcomponent_as_ical_string(compresult));
+		dprintf("Component found\n%s\n", icalcomponent_as_ical_string(compresult));
 
 	}
 	if (compresult)
@@ -501,11 +516,12 @@ GNOKII_API int gn_ical2todo(FILE *f, gn_todo *ctodo, int id)
 	if (id < 1)
 		id = 1;
 
-	/* interate through the component */
+	/* iterate through the component */
 	iterate_cal(comp, 0, &id, &compresult, ICAL_VTODO_COMPONENT);
 
 	if (!compresult) {
-		fprintf(stderr, _("No component found.\n"));
+		dprintf("No component found.\n");
+		return GN_ERR_EMPTYLOCATION;
 	} else {
 		const char *priostr = NULL;
 		icalproperty *priority = icalcomponent_get_first_property(compresult, ICAL_PRIORITY_PROPERTY);
@@ -517,7 +533,7 @@ GNOKII_API int gn_ical2todo(FILE *f, gn_todo *ctodo, int id)
 		/* priority */
 		ctodo->priority = icalproperty_get_priority((const icalproperty *)priority);
 
-		fprintf(stderr, _("Component found\n%s\n"), icalcomponent_as_ical_string(compresult));
+		dprintf("Component found\n%s\n", icalcomponent_as_ical_string(compresult));
 	}
 	icalcomponent_free(compresult);
 	icalcomponent_free(comp);
@@ -542,7 +558,7 @@ static int iterate_cal(icalcomponent *parent, int depth, int *count, icalcompone
 	if (!parent)
 		return 0;
 
-	/* we need to use interators because we're calling iterate_cal recursively */
+	/* we need to use iterators because we're calling iterate_cal recursively */
 	compiter = icalcomponent_begin_component(parent, ICAL_ANY_COMPONENT);
 	comp = icalcompiter_deref(&compiter);
 	for (; comp && *count; comp = icalcompiter_next(&compiter)) {
