@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii.h,v 1.103 2006/08/10 22:17:40 pkot Exp $
+  $Id: gnokii.h,v 1.112 2007/07/10 15:37:02 pkot Exp $
 
   G N O K I I
 
@@ -65,9 +65,13 @@ typedef unsigned char uint8_t;
 #endif
 
 /* if you change those numbers, change them in common/Makefile as well! */
-#define LIBGNOKII_VERSION_STRING "3.0.0"
+/* LIBGNOKII_VERSION_MAJOR = GNOKII_MAJOR - GNOKII_MINOR
+ * LIBGNOKII_VERSION_MINOR = GNOKII_MINOR
+ * LIBGNOKII_VERSION_RELEASE = GNOKII_RELEASE
+ */
+#define LIBGNOKII_VERSION_STRING "3.2.0"
 #define LIBGNOKII_VERSION_MAJOR 3
-#define LIBGNOKII_VERSION_MINOR 0
+#define LIBGNOKII_VERSION_MINOR 2
 #define LIBGNOKII_VERSION_RELEASE 0
 #define LIBGNOKII_MAKE_VERSION( a,b,c ) (((a) << 16) | ((b) << 8) | (c))
 
@@ -95,7 +99,7 @@ struct gn_statemachine;
 /* Get the current runtime version (LIBGNOKII_VERSION) of libgnokii - see LIBGNOKII_MAKE_VERSION() */
 GNOKII_API unsigned int gn_lib_version();
 
-/* free any ressources which were used by libgnokii */
+/* free any resources which were used by libgnokii */
 GNOKII_API void gn_lib_library_free( void );
 
 /* return last error code from functions below */
@@ -107,7 +111,7 @@ GNOKII_API gn_error gn_lib_lasterror( struct gn_statemachine *state );
 GNOKII_API gn_error gn_lib_phoneprofile_load_from_file( const char *configfile, const char *configname, struct gn_statemachine **state );
 GNOKII_API gn_error gn_lib_phoneprofile_load( const char *configname, struct gn_statemachine **state );
 
-/* Free any structures and ressources which were loaded in this state machine. */
+/* Free any structures and resources which were loaded in this state machine. */
 GNOKII_API gn_error gn_lib_phoneprofile_free( struct gn_statemachine **state );
 
 /* opens connection to the phone. Necessary locking will be made */
@@ -178,7 +182,7 @@ GNOKII_API gn_error gn_lib_set_pb_subentry    ( struct gn_statemachine *state, c
 GNOKII_API gn_error gn_lib_phonebook_write_entry( struct gn_statemachine *state,
         const gn_memory_type memory_type, const int index );
 
-
+GNOKII_API char *gn_subentrytype2string(gn_phonebook_entry_type entry_type, gn_phonebook_number_type number_type);
 
 /* standard helper functions */
 
@@ -195,6 +199,11 @@ GNOKII_API const char *gn_lib_get_supported_connection( const int num );
    The state variable will be initialized if a phone was found. */
 GNOKII_API gn_error gn_lib_search_one_connected_phone( struct gn_statemachine **state );
 
+/* use this function to check whether the connection type is enabled
+   0 - not supported
+   1 - supported */
+GNOKII_API int gn_lib_is_connectiontype_supported(gn_connection_type ct);
+
 /* Misc */
 GNOKII_API gn_memory_type gn_str2memory_type(const char *s);
 GNOKII_API const char *gn_memory_type2str(gn_memory_type mt);
@@ -251,7 +260,7 @@ GNOKII_API gn_error gn_cfg_file_read(const char *filename) GNOKII_DEPRECATED;
 GNOKII_API gn_error gn_cfg_memory_read(const char **lines) GNOKII_DEPRECATED;
 /* Read the config from the standard $HOME/.gnokiirc or /etc/gnokiirc locations */
 GNOKII_API gn_error gn_cfg_read_default() GNOKII_DEPRECATED;
-/* free any ressources which were used by libgnokii */
+/* free any resources which were used by libgnokii */
 GNOKII_API void gn_cfg_free_default() GNOKII_DEPRECATED;
 /* Use phone_iname section for the communication. Default is the global section */
 GNOKII_API gn_error gn_cfg_phone_load(const char *iname, struct gn_statemachine *state) GNOKII_DEPRECATED;
