@@ -1,6 +1,6 @@
 /*
 
-  $Id: xgnokii.c,v 1.88 2006/08/15 15:04:59 dforsi Exp $
+  $Id: xgnokii.c,v 1.92 2007/06/06 14:58:29 dforsi Exp $
   
   X G N O K I I
 
@@ -33,7 +33,9 @@
 
 #include "config.h"
 #ifdef HAVE_ASPRINTF
-#  define _GNU_SOURCE
+#  ifndef _GNU_SOURCE 
+#    define _GNU_SOURCE 1
+#  endif
 #  include <stdio.h>
 #endif
 
@@ -54,7 +56,6 @@
 #endif
 
 #ifndef WIN32
-#  include <unistd.h>		/* for usleep */
 #  include <signal.h>
 #else
 #  include <windows.h>
@@ -816,7 +817,7 @@ inline void GUI_HideAbout(void)
 }
 
 
-void MainExit(void)
+static void _MainExit(void)
 {
 	PhoneEvent *e = (PhoneEvent *) g_malloc(sizeof(PhoneEvent));
 
@@ -827,6 +828,17 @@ void MainExit(void)
 	gtk_main_quit();
 }
 
+void MainExit(gchar *ermsg)
+{
+	if (ermsg) {
+		gtk_label_set_text(GTK_LABEL(infoDialog.text), ermsg);
+		gtk_widget_show_now(infoDialog.dialog);
+		GUI_Refresh();
+		sleep(10);
+	}
+	_MainExit();
+}
+
 
 static void ShowMenu(GdkEventButton * event)
 {
@@ -883,7 +895,7 @@ static gint ButtonPressEvent(GtkWidget * widget, GdkEventButton * event)
 			if (GUI_ContactsIsChanged())
 				GUI_QuitSaveContacts();
 			else
-				MainExit();
+				MainExit(NULL);
 		} else if ((event->x >= 180 && event->x <= 195 &&
 			    event->y >= 30 && event->y <= 50) ||
 			   (event->x >= 185 && event->x <= 215 &&
@@ -1322,7 +1334,7 @@ static void OkEditSMSSetDialog(GtkWidget * w, gpointer data)
 		gtk_entry_get_text(GTK_ENTRY(configDialogData.sms.number)),
 		GN_BCD_STRING_MAX_LENGTH);
 	configDialogData.sms.smsSetting[GPOINTER_TO_INT(GTK_CLIST(SMSClist)->selection->data)].
-	    smsc.number[GN_BCD_STRING_MAX_LENGTH]
+	    smsc.number[GN_BCD_STRING_MAX_LENGTH - 1]
 	    = '\0';
 
 	configDialogData.sms.smsSetting[GPOINTER_TO_INT(GTK_CLIST(SMSClist)->selection->data)].
@@ -2225,7 +2237,7 @@ static void TopLevelWindow(void)
 
 	gtk_widget_shape_combine_mask(GUI_MainWindow, mask, 0, 0);
 
-	gtk_signal_connect(GTK_OBJECT(GUI_MainWindow), "destroy", GTK_SIGNAL_FUNC(MainExit), NULL);
+	gtk_signal_connect(GTK_OBJECT(GUI_MainWindow), "destroy", GTK_SIGNAL_FUNC(_MainExit), NULL);
 
 	Menu = CreateMenu();
 	OptionsDialog = CreateOptionsDialog();
@@ -2346,6 +2358,10 @@ static void ReadConfig(void)
 		xgnokiiConfig.bindir = gn_lib_cfg_get("gnokiid", "bindir");
 	if (!xgnokiiConfig.bindir)
 		xgnokiiConfig.bindir = g_strdup("/usr/local/sbin");
+	if (gn_lib_cfg_get("xgnokii", "allow_breakage"))
+		xgnokiiConfig.allowBreakage = atoi(gn_lib_cfg_get("xgnokii", "allow_breakage"));
+	else
+		xgnokiiConfig.allowBreakage = 0;
 
 	max_phonebook_number_length = max_phonebook_sim_number_length =
 	    GN_PHONEBOOK_NUMBER_MAX_LENGTH;
@@ -2394,8 +2410,6 @@ int main(int argc, char *argv[])
 	textdomain("gnokii");
 #endif
 
-	(void) gtk_set_locale();
-
 	gtk_init(&argc, &argv);
 
 	gn_elog_handler = NULL;
